package com.kz.grbl;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;

import org.jzy3d.chart.Chart;
import org.jzy3d.chart.controllers.mouse.camera.NewtCameraMouseController;
import org.jzy3d.chart.controllers.mouse.picking.NewtMousePickingPan2dController;
import org.jzy3d.chart.controllers.thread.camera.CameraThreadController;
import org.jzy3d.chart.factories.AWTChartComponentFactory;
import org.jzy3d.chart.factories.IChartComponentFactory.Toolkit;
import org.jzy3d.colors.Color;
import org.jzy3d.maths.Coord3d;
import org.jzy3d.picking.IObjectPickedListener;
import org.jzy3d.picking.PickingSupport;
import org.jzy3d.plot3d.primitives.AbstractDrawable;
import org.jzy3d.plot3d.primitives.LineStrip;
import org.jzy3d.plot3d.primitives.Point;
import org.jzy3d.plot3d.primitives.pickable.PickablePoint;
import org.jzy3d.plot3d.primitives.pickable.PickablePolygon;
import org.jzy3d.plot3d.rendering.canvas.CanvasNewtAwt;
import org.jzy3d.plot3d.rendering.canvas.Quality;
import org.jzy3d.plot3d.transform.Rotate;
import org.jzy3d.plot3d.transform.Transform;
import org.jzy3d.plot3d.transform.Translate;
import org.openide.util.RequestProcessor;
import org.openide.util.Task;
import org.openide.util.TaskListener;
import org.openide.windows.TopComponent;

import com.jogamp.newt.event.KeyEvent;
import com.jogamp.newt.event.KeyListener;
import com.jogamp.newt.event.MouseEvent;

/*
 * Copyright 2014 Nils Hoffmann.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// http://jogamp.org/wiki/index.php/Using_JOGL_in_Java_Web_Start
// http://mojo.codehaus.org/webstart/webstart-maven-plugin/
/**
 * Top component which displays a JZY3D demo.
 */
public final class SurfaceDemo2 extends TopComponent {

	public Chart chart;
	public CanvasNewtAwt canvas;
	private CameraThreadController ctc;
	private AtomicBoolean chartCreated = new AtomicBoolean(false);
	public static SurfaceDemo2 instance;
	SubClassNewtCameraMouseController camMouse;
	private List<PickablePolygon> polygons;
	private Cylinder bar = null;
	public boolean mouseDown = false;
	MyPickablePoint lastClickedPoint = null;
	Utils utils;

	JTextField angle = null;

	public SurfaceDemo2() {
		setOpaque(true);
		initComponents();
		setName("sdfsdf");
		setToolTipText("sdfsdf");
		createChart();
		instance = this;
	}

	public SurfaceDemo2 getInstance() {
		if (instance == null)
			instance = this;
		return instance;
	}

	@Override
	protected void componentOpened() {
		super.componentOpened();
		System.out.println("Opened");
		createChart();
	}

	@Override
	protected void componentClosed() {
		super.componentClosed();
		System.out.println("Closed");
		if (chartCreated.get() && chart != null) {
			try {
				// dispose chart resources
				chart.dispose();
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				// cleanup
				chart = null;
				ctc = null;
				remove(canvas);
				// dispose of canvas and native resources
				canvas.dispose();
				chartCreated.set(false);
			}
		}
	}

	@Override
	protected void componentShowing() {
		super.componentShowing();
		System.out.println("Showing");
		if (chartCreated.get()) {
			if (ctc != null) {
				if (toggleAnimation.isSelected()) {
					// reenable animation if animation is toggled and component
					// is set to visible
					ctc.start();
				}
			}
		}
	}

	@Override
	protected void componentHidden() {
		super.componentHidden();
		System.out.println("Hidden");
		if (chartCreated.get()) {
			if (ctc != null) {
				// stop animation if top component is not visible
				ctc.stop();
			}
		}
	}

	@Override
	protected void componentActivated() {
		super.componentActivated();
		System.out.println("Activated");
		if (chartCreated.get()) {
			if (chart != null) {
				CanvasNewtAwt cna = (CanvasNewtAwt) chart.getCanvas();
				cna.setSize(getSize());
				// need to update complete component tree
				invalidate();
				getTopLevelAncestor().invalidate();
				getTopLevelAncestor().revalidate();
			}
		}
	}

	private void createChart() {
		if (chartCreated.compareAndSet(false, true)) {
			Runnable worker = new Runnable() {

				@Override
				public void run() {
					// ProgressHandle ph =
					// ProgressHandleFactory.createHandle("Opening Jzy3D View!");
					// ph.start();
					// ph.switchToIndeterminate();
					System.out.println("Creating chart! Thread: " + Thread.currentThread().getName());
					try {
						AWTChartComponentFactory accf = new AWTChartComponentFactory();
						//ChartComponentFactory accf = new ChartComponentFactory();

						final Chart chart = accf.newChart(Quality.Advanced, Toolkit.newt.name());
						chart.getView().setMaximized(true);
						canvas = (CanvasNewtAwt) chart.getCanvas();
						final CameraThreadController ctc = new CameraThreadController(chart);
						// signature change in latest upstream jogl causes
						// java.lang.NoSuchMethodError:
						// com.jogamp.newt.event.MouseEvent.getClickCount()I
						// NewtCameraMouseController camMouse = new
						// NewtCameraMouseController(chart);
						// WorkaroundNewtCameraMouseController camMouse = new
						// WorkaroundNewtCameraMouseController(chart);
						// enablePicking(points, chart);
						// // Create a surface drawing that function
						// // Define a function to plot
						// Mapper mapper = new Mapper() {
						// public double f(double x, double y) {
						// return 10 * Math.sin(x / 10) * Math.cos(y / 20) * x;
						// }
						// };
						// // Define range and precision for the function to
						// plot
						// Range range = new Range(-150, 150);
						// int steps = 50;
						// Shape surface = Builder.buildOrthonormal(new
						// OrthonormalGrid(range, steps, range, steps),
						// mapper);
						// surface.setColorMapper(new ColorMapper(new
						// ColorMapRainbow(), surface.getBounds().getZmin(),
						// surface.getBounds().getZmax(), new Color(1, 1, 1,
						// .5f)));
						// surface.setFaceDisplayed(true);
						// surface.setWireframeDisplayed(false);
						// surface.setWireframeColor(Color.BLACK);

						// //////////////////

						// chart = new Chart(Quality.Intermediate,
						// Toolkit.newt.name());

						String filePath = null;
						if (OSValidator.isUnix())
							filePath = "/home/klemen/googledrive/TerasaBoris/grbl";
						else
							filePath = "c:\\Users\\klemen\\Google Drive\\TerasaBoris\\grbl";

						Path path = FileSystems.getDefault().getPath(filePath, "data.csv");
						List<String> lines;
						lines = Files.readAllLines(path, Charset.forName("UTF-8"));
						utils = new Utils();

						ArrayList<String> line = new ArrayList<String>();

						//Point previousPolyPoint = null;
						LineStrip ls_p = null;
						int previousSurfaceNo = -1;
						int previousEdgeNo = -1;
						int pointId = -1;
						for (String string : lines) {
							line.add(string);
							String splitted[] = string.split(";");
							int surfaceNo = Integer.valueOf(splitted[0].split("_")[1]);
							int edgeNo = Integer.valueOf(splitted[0].split("_")[2]);
							int colorNo = surfaceNo % Color.COLORS.length;

							if (splitted[0].startsWith("EDGE")) {

								Float x1 = Float.valueOf(splitted[1].replace(",", "."));
								Float y1 = Float.valueOf(splitted[2].replace(",", "."));
								Float z1 = Float.valueOf(splitted[3].replace(",", "."));
								Coord3d p1 = new Coord3d(x1, y1, z1);
								int id1 = pointId++;
								MyPickablePoint mp1 = utils.createOrFindMyPickablePoint(id1, surfaceNo, edgeNo, p1,
										Color.COLORS[colorNo], 5);

								Float x2 = Float.valueOf(splitted[4].replace(",", "."));
								Float y2 = Float.valueOf(splitted[5].replace(",", "."));
								Float z2 = Float.valueOf(splitted[6].replace(",", "."));
								Coord3d p2 = new Coord3d(x2, y2, z2);
								int id2 = pointId++;
								MyPickablePoint mp2 = utils.createOrFindMyPickablePoint(id2, surfaceNo, edgeNo, p2,
										Color.COLORS[colorNo], 5);

								LineStrip ls = new LineStrip();
								ls.add(mp1);
								ls.add(mp2);
								ls.setWireframeColor(Color.COLORS[colorNo]);
								chart.getScene().getGraph().add(ls);
							} else if (splitted[0].startsWith("POLY")) {
								if (surfaceNo != previousSurfaceNo || edgeNo != previousEdgeNo) {
									if (ls_p != null) {
										ls_p.setWireframeColor(Color.COLORS[colorNo]);
										chart.getScene().getGraph().add(ls_p);
									}
									ls_p = new LineStrip();
								}

								Float x = Float.valueOf(splitted[1].replace(",", "."));
								Float y = Float.valueOf(splitted[2].replace(",", "."));
								Float z = Float.valueOf(splitted[3].replace(",", "."));
								Coord3d coord = new Coord3d(x.floatValue(), y.floatValue(), z.floatValue());

								int id = pointId++;
								MyPickablePoint pickablePoint = utils.createOrFindMyPickablePoint(id, surfaceNo,
										edgeNo, coord, Color.COLORS[colorNo], 5);
								if (ls_p != null) {
									ls_p.add(pickablePoint);
								}
								previousSurfaceNo = surfaceNo;
								previousEdgeNo = edgeNo;
							}
						}

						// Creates the 3d object
						chart.getScene().getGraph().add(utils.points);
						// Range xrange =
						// chart.getScene().getGraph().getBounds().getXRange();
						// Range yrange =
						// chart.getScene().getGraph().getBounds().getYRange();
						// Range zrange =
						// chart.getScene().getGraph().getBounds().getZRange();
						// chart.getScene().getGraph().getBounds().setXmax(xrange.getMax());
						// chart.getScene().getGraph().getBounds().setXmin(xrange.getMin());
						// chart.getScene().getGraph().getBounds().setYmax(yrange.getMax());
						// chart.getScene().getGraph().getBounds().setYmin(yrange.getMin());
						// chart.getScene().getGraph().getBounds().setZmax(zrange.getMax());
						// chart.getScene().getGraph().getBounds().setZmin(zrange.getMin());
						// chart.getView().zoomX(1);
						// chart.getView().zoomY(1);
						// chart.getView().zoomZ(1);
						chart.getView().setSquared(false);

						camMouse = new SubClassNewtCameraMouseController(chart);
						enablePicking(utils.points, polygons, chart);
						System.out.println("Thread: " + Thread.currentThread().getName());
						// Using JOGL text renderer with or without a text style

						// ITextRenderer renderer2 = new JOGLTextRenderer(new
						// DefaultTextStyle(java.awt.Color.RED));
						// t6 = new DrawableTextWrapper("MyText", new
						// Coord3d(1,1,1), Color.BLACK, renderer2);
						// chart.getScene().getGraph().add( t6 );

						// //////////////////

						SwingUtilities.invokeLater(new Runnable() {

							@Override
							public void run() {
								setChart(chart);
								canvas.setMinimumSize(getMinimumSize());
								canvas.setMaximumSize(getMaximumSize());
								canvas.setPreferredSize(getPreferredSize());

								canvas.addKeyListener(new KeyListener() {

									@Override
									public void keyReleased(KeyEvent e) {
										// TODO Auto-generated method stub

									}

									@Override
									public void keyPressed(KeyEvent e) {
										// TODO Auto-generated method stub
										System.out.println(e.getKeyCode());
										if (e.getKeyCode() == 150 || e.getKeyCode() == 152) {
											double angleDelta = 0;
											if (e.getKeyCode() == 150) {
												System.out.println("UP");
												angleDelta = 1;
											}

											if (e.getKeyCode() == 152) {
												System.out.println("DOWN");
												angleDelta = -1;
											}
											Double value = Double.valueOf(angle.getText());
											value = value + angleDelta;
											angle.setText(value.toString());

											Rotate rotate = new Rotate(angleDelta, new Coord3d(0, 1, 0));
											Transform t = new Transform();
											t.add(rotate);
											Iterator<AbstractDrawable> it = chart.getScene().getGraph().getAll()
													.iterator();
											while (it.hasNext()) {
												AbstractDrawable dr = it.next();
												if (dr.getClass().getName().equals("com.kz.grbl.MyPickablePoint"))
													dr.applyGeometryTransform(t);
												else
													System.out.println(dr.getClass().getName());
											}
										}

										chart.render();

									}
								});

								add(canvas, BorderLayout.CENTER);
								// setCameraThreadController(ctc);
								// if (toggleAnimation.isSelected()) {
								// ctc.start();
								// }
								// canvas.getAnimator().stop();

								// need to update complete component tree
								invalidate();
								getTopLevelAncestor().invalidate();
								getTopLevelAncestor().revalidate();
							}
						});

					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} finally {
						// ph.finish();
					}
				}

			};
			Task t = RequestProcessor.getDefault().post(worker);
			t.addTaskListener(new TaskListener() {

				@Override
				public void taskFinished(Task task) {
					if (task.isFinished()) {
						System.out.println("Attention!");
					}
				}

			});
		} else {
			System.out.println("Chart already created!");
		}
	}

	private void setChart(Chart chart) {
		this.chart = chart;
	}

	private void setCameraThreadController(CameraThreadController controller) {
		this.ctc = controller;
	}

	static class WorkaroundNewtCameraMouseController extends NewtCameraMouseController {

		public WorkaroundNewtCameraMouseController(Chart chart) {
			super(chart);
		}

		/**
		 * Compute zoom
		 */
		@Override
		public void mouseWheelMoved(MouseEvent e) {
			stopThreadController();
			zoomZ(1 + (e.getRotation()[0] * e.getRotationScale() / 10.0f));
		}

		@Override
		public boolean handleSlaveThread(MouseEvent e) {
			if (e.getClickCount() == 2) {
				if (threadController != null) {
					threadController.start();
					return true;
				}
			}
			if (threadController != null) {
				threadController.stop();
			}
			return false;
		}

	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		jToolBar1 = new JToolBar();
		toggleAnimation = new JToggleButton();

		setMinimumSize(new java.awt.Dimension(320, 240));
		setPreferredSize(new java.awt.Dimension(640, 480));
		setLayout(new java.awt.BorderLayout());

		jToolBar1.setRollover(true);

		org.openide.awt.Mnemonics.setLocalizedText(toggleAnimation, "Animate"); // NOI18N
		toggleAnimation.setFocusable(false);
		toggleAnimation.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		toggleAnimation.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
		toggleAnimation.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				toggleAnimationActionPerformed(evt);
			}
		});
		jToolBar1.add(toggleAnimation);

		angle = new JTextField();
		jToolBar1.add(angle);
		angle.setText("0.0");

		JButton setOrigin = new JButton();
		setOrigin.setText("SetOrigin");
		setOrigin.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				String text = JOptionPane.showInputDialog("Enter 3d point in form x,y,z", "0,0,0");

				float x = Float.valueOf(text.split(",")[0]);
				float y = Float.valueOf(text.split(",")[1]);
				float z = Float.valueOf(text.split(",")[2]);

				float deltaX = x - lastClickedPoint.xyz.x;
				float deltaY = y - lastClickedPoint.xyz.y;
				float deltaZ = z - lastClickedPoint.xyz.z;

				Translate translate = new Translate(new Coord3d(deltaX, deltaY, deltaZ));
				Transform t = new Transform();
				t.add(translate);
				Iterator<AbstractDrawable> it = chart.getScene().getGraph().getAll().iterator();
				while (it.hasNext()) {
					AbstractDrawable dr = it.next();
					dr.applyGeometryTransform(t);
				}
				chart.render();
			}
		});
		jToolBar1.add(setOrigin);

		JButton startGCode = new JButton();
		startGCode.setText("StartGCode");
		startGCode.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				SurfaceDemo2.instance.startGCode();
			}
		});
		jToolBar1.add(startGCode);
		add(jToolBar1, java.awt.BorderLayout.PAGE_START);

	}// </editor-fold>//GEN-END:initComponents

	private void toggleAnimationActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_toggleAnimationActionPerformed
		if (ctc != null) {
			if (toggleAnimation.isSelected()) {
				ctc.start();
			} else {
				ctc.stop();
			}
		}
	}// GEN-LAST:event_toggleAnimationActionPerformed

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JToolBar jToolBar1;
	private javax.swing.JToggleButton toggleAnimation;

	// End of variables declaration//GEN-END:variables
	void writeProperties(java.util.Properties p) {
		// better to version settings since initial version as advocated at
		// http://wiki.apidesign.org/wiki/PropertyFiles
		p.setProperty("version", "1.0");
		// TODO store your settings
	}

	void readProperties(java.util.Properties p) {
		String version = p.getProperty("version");
		// TODO read your settings according to their version
	}

	public static void main(String args[]) {
		JFrame frame = new JFrame();
		frame.add(new SurfaceDemo2());
		frame.setSize(600, 600);
		frame.setVisible(true);
	}

	private void enablePicking(List<MyPickablePoint> points, List<PickablePolygon> polygons, Chart chart) {
		NewtMousePickingPan2dController<?, ?> mousePicker = new NewtMousePickingPan2dController<>(chart, 5);
		PickingSupport picking = mousePicker.getPickingSupport();

		for (PickablePoint p : points) {
			picking.registerPickableObject(p, p);
		}
		if (polygons != null)
			for (PickablePolygon pPolygon : polygons) {
				picking.registerPickableObject(pPolygon, pPolygon);
			}

		picking.addObjectPickedListener(new IObjectPickedListener() {
			@Override
			public void objectPicked(List<?> picked, PickingSupport ps) {
				if (picked.size() > 0 && (System.currentTimeMillis() - camMouse.clickTimeMillis < 200)) {
					System.out.println("Size: " + picked.size());
					for (int i = 0; i < picked.size(); i++) {
						if (picked.get(i).getClass().getName().equals("com.kz.grbl.MyPickablePoint")) {
							((Point) picked.get(i)).setBoundingBoxColor(Color.BLACK);
							((Point) picked.get(i)).setBoundingBoxDisplayed(true);
							MyPickablePoint mp = ((MyPickablePoint) picked.get(i));
							System.out.println(mp.toString());
							addCylinder(mp.xyz);
							lastClickedPoint = mp;
						} else if (picked.get(i).getClass().getName()
								.equals("org.jzy3d.plot3d.primitives.pickable.PickablePolygon")) {
							System.out.println(picked.get(i).getClass().getName());
						} else {
							System.out.println(picked.get(i).getClass().getName());
						}

					}

				}
			}
		});
	}

	/**
	 * Calculate the angle at the vertex between two rays formed by three 3d
	 * points.
	 * 
	 * @param a
	 *            A 3d point.
	 * @param vertex
	 *            The vertex point.
	 * @param b
	 *            A 3d point.
	 * 
	 * @return The angle, from 0 to 2 * PI, in radians.
	 */
	public static double calculateAngle(Coord3d a, Coord3d vertex, Coord3d b) {
		double lengthAB = Math.sqrt(Math.pow(a.x - b.x, 2) + Math.pow(a.y - b.y, 2) + Math.pow(a.z - b.z, 2));
		double lengthAV = Math.sqrt(Math.pow(a.x - vertex.x, 2) + Math.pow(a.y - vertex.y, 2)
				+ Math.pow(a.z - vertex.z, 2));
		double lengthBV = Math.sqrt(Math.pow(b.x - vertex.x, 2) + Math.pow(b.y - vertex.y, 2)
				+ Math.pow(b.z - vertex.z, 2));
		double numerator = Math.pow(lengthAV, 2) + Math.pow(lengthBV, 2) - Math.pow(lengthAB, 2);
		double denominator = 2 * lengthAV * lengthBV;
		return Math.acos(numerator / denominator);
	}

	public void addCylinder(Coord3d coord3d) {
		System.out.println("Thread: " + Thread.currentThread().getName());
		
		Color color = Color.RED; // mapper.getColor(new Coord3d(0,0,height));
		color.a = 0.55f;

		if (bar != null) {
			bar.clean();
		}

		bar = new Cylinder(this.chart.getScene().getGraph());
		double height = 0.7;
		bar.setData(coord3d, (float) height, (float) 0.1, 15, 5, color);
		bar.setWireframeDisplayed(true);
		bar.setWireframeColor(Color.BLACK);
		chart.getScene().getGraph().add(bar);

	}

	static class SubClassNewtCameraMouseController extends NewtCameraMouseController {
		public static long clickTimeMillis;

		public SubClassNewtCameraMouseController(Chart chart) {
			super(chart);

		}

		@Override
		public void mouseClicked(MouseEvent e) {
			super.mouseClicked(e);
			clickTimeMillis = System.currentTimeMillis();
		}

		@Override
		public void mouseDragged(MouseEvent e) {
			super.mouseDragged(e);
			// System.out.println("" + e.getModifiers());
			boolean isLeftDown1 = (e.getModifiers() & MouseEvent.BUTTON1_MASK) == MouseEvent.BUTTON1_MASK;
			boolean isLeftDown2 = isLeftDown(e);
			// System.out.println("" + isLeftDown1 + " " + isLeftDown2); // true
			// false
			// !!!
		}

	}

	public void startGCode() {
		MyPickablePoint myPoint = SurfaceDemo2.instance.lastClickedPoint;
		Cylinder cyl;

		ArrayList<Coord3d> path = new ArrayList<Coord3d>();

		System.out.println("----------");
		System.out.println(myPoint.id);

		if (myPoint != null) {
			int surfaceNo = myPoint.surfaceNo;

			Iterator<AbstractDrawable> it = SurfaceDemo2.instance.chart.getScene().getGraph().getAll().iterator();
			while (it.hasNext()) {
				AbstractDrawable dr = it.next();
				if (dr.getClass().getName().equals("org.jzy3d.plot3d.primitives.LineStrip")) {
					org.jzy3d.plot3d.primitives.LineStrip ls = (org.jzy3d.plot3d.primitives.LineStrip) dr;
					for (Point mpPoint : ls.getPoints()) {
						MyPickablePoint mpPoint1 = (MyPickablePoint) mpPoint;
						if (mpPoint1.surfaceNo == surfaceNo) {
							System.out.println(mpPoint1.id + " " + mpPoint.xyz.x + " " + mpPoint.xyz.y + " "
									+ mpPoint.xyz.z);
							path.add(mpPoint1.xyz);
						}
					}
				}
				// else System.out.println(dr.getClass().getName());
			}
		}
		SwingUtilities.invokeLater(new Visualizer(path));
	}

	private class Visualizer implements Runnable {
		ArrayList<Coord3d> path = new ArrayList<Coord3d>();

		public Visualizer(ArrayList<Coord3d> pts) {
			this.path = pts;

		}

		@Override
		public void run() {
			// TODO Auto-generated method stub
			System.out.println("Thread: " + Thread.currentThread().getName());
			for (Coord3d coord : path) {
				Color color = Color.BLUE; // mapper.getColor(new
											// Coord3d(0,0,height));
				Cylinder cyl = new Cylinder(SurfaceDemo2.instance.chart.getScene().getGraph());
				double height = 0.7;
				cyl.setData(coord, (float) height, (float) 0.1, 15, 5, color);
				cyl.setWireframeDisplayed(true);
				cyl.setWireframeColor(Color.BLACK);

				// SurfaceDemo2.instance.addCylinder(coord);
				SurfaceDemo2.instance.canvas.getAnimator().pause();
				SurfaceDemo2.instance.chart.getScene().add(cyl);
				SurfaceDemo2.instance.canvas.getAnimator().resume();
				// System.out.println("Refreshing...");

				SurfaceDemo2.instance.chart.getScene().add(new PickablePoint(new Coord3d(
						(float)2*Math.random(),
						(float)2*Math.random(),
						(float)2*Math.random()),Color.MAGENTA,(float)0.5
						));
				
				try {
					//SurfaceDemo2.instance.canvas.getAnimator().pause();
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

	}

}